/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2012 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;

import de.javagl.swogl.geometry.Ray;
import de.javagl.swogl.scene.Camera;
import de.javagl.swogl.scene.View;


/**
 * Implementation of a {@link PickingRayComputer} that computes the
 * picking {@link Ray} based on a given {@link Camera} and 
 * {@link View}.
 */
public final class DefaultPickingRayComputer implements PickingRayComputer
{
    /**
     * The backing view
     */
    private final View view;
    
    /**
     * The backing camera
     */
    private final Camera camera;
    
    /**
     * Creates a new picking ray computer that computes the
     * picking ray based on the given camera and view.
     * 
     * @param camera The camera to operate on.
     * @param view The view to operate on.
     */
    public DefaultPickingRayComputer(Camera camera, View view)
    {
        this.camera = camera;
        this.view = view;
    }
    
    @Override
    public Ray computePickingRay(int x, int y)
    {
        return PickingUtils.computePickingRay(camera, view, x, y);
    }
}
