/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2012 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;

import java.awt.Point;

import javax.swing.JComponent;


/**
 * This class stores information about a picked component. Instances
 * of this class may be obtained from an {@link IntersectionInfo}.
 */
public class PickingInfo
{
    /**
     * The component that has been picked
     */
    private final JComponent component;
    
    /**
     * The point where the component was hit
     */
    private final Point point;
    
    /**
     * Creates a new PickingInfo from the given parameters
     * 
     * @param component The component that has been picked
     * @param point The position where the component was hit
     */
    PickingInfo(JComponent component, Point point)
    {
        this.component = component;
        this.point = new Point(point);
    }

    /**
     * Returns the component that has been picked
     * 
     * @return The component that has been picked
     */
    public JComponent getComponent()
    {
        return component;
    }

    /**
     * Returns a copy of the point where the component was hit
     * 
     * @return A copy of the point where the component was hit
     */
    public Point getPoint()
    {
        return new Point(point);
    }

    /**
     * Returns a String representation of this PickingInfo
     * 
     * @return A String representation of this PickingInfo
     */
    @Override
    public String toString()
    {
        return "PickingInfo[" +
        	"point="+point+","+
            "component="+component+"]";
    }
    
}
