/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2012 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;

import javax.swing.JComponent;

import de.javagl.swogl.geometry.Geometry;

/**
 * Factory methods for creating SwoglComponents.
 */
public class SwoglComponents
{

    /**
     * Creates a new SwoglComponent that contains the given Swing component.
     * The geometry and texture of the SwoglComponent will have the 
     * size that is returned as the preferred size of the given component.
     * 
     * @param component The component
     * @return The SwoglComponent
     */
    public static SwoglComponent create(JComponent component)
    {
        return new DefaultSwoglComponent(component);
    }

    /**
     * Creates a new SwoglComponent that contains the given Swing component.
     * The geometry and texture of the SwoglComponent will have the 
     * given size.
     * 
     * @param component The component
     * @param w The width
     * @param h The height
     * @return The SwoglComponent
     */
    public static SwoglComponent create(
        JComponent component, int w, int h)
    {
        return new DefaultSwoglComponent(component, w, h);
    }

    /**
     * Create a new SwoglComponent that contains the given Swing component
     * and consists of the given geometry and texture.
     * 
     * @param geometry The geometry
     * @param texture The texture
     * @param component The component
     * @return The SwoglComponent
     */
    public static SwoglComponent create(
        Geometry geometry, Texture texture, JComponent component)
    {
        return new DefaultSwoglComponent(geometry, texture, component);
    }

    /**
     * Private constructor to prevent instantiation
     */
    private SwoglComponents()
    {
    }
}
