/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2012 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.geometry;

import javax.vecmath.Tuple3f;

/**
 * Interface describing a box
 */
public interface Box
{
    /**
     * Returns the minimum x value of this box
     * 
     * @return The minimum x value of this box
     */
    float getMinX();

    /**
     * Returns the minimum y value of this box
     * 
     * @return The minimum y value of this box
     */
    float getMinY();

    /**
     * Returns the minimum z value of this box
     * 
     * @return The minimum z value of this box
     */
    float getMinZ();

    /**
     * Returns the maximum x value of this box
     * 
     * @return The maximum x value of this box
     */
    float getMaxX();

    /**
     * Returns the maximum y value of this box
     * 
     * @return The maximum y value of this box
     */
    float getMaxY();

    /**
     * Returns the maximum z value of this box
     * 
     * @return The maximum z value of this box
     */
    float getMaxZ();

    /**
     * Stores in the given tuple the minimum point of this box
     * 
     * @param min The minimum point of this box
     */
    void getMin(Tuple3f min);

    /**
     * Stores in the given tuple the maximum point of this box
     * 
     * @param max The maximum point of this box
     */
    void getMax(Tuple3f max);

    /**
     * Returns the size of this box in x direction
     *  
     * @return The size of this box in x direction
     */
    float getSizeX();

    /**
     * Returns the size of this box in y direction
     *  
     * @return The size of this box in y direction
     */
    float getSizeY();

    /**
     * Returns the size of this box in z direction
     *  
     * @return The size of this box in z direction
     */
    float getSizeZ();

    /**
     * Stores in the given tuple the center of this box
     * 
     * @param center The center of this box
     */
    void getCenter(Tuple3f center);

}