/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2012 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.layout;

import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JComponent;

import de.javagl.swogl.interaction.Control;


/**
 * Implementation of a {@link Control} for a {@link WheelLayout3D}.
 */
public class WheelLayoutControl implements Control
{
    /**
     * The {@link WheelLayout3D} this control operates on
     */
    private WheelLayout3D wheelLayout;

    /**
     * The listener that will be attached to the component
     */
    class Listener implements MouseWheelListener
    {
        @Override
        public void mouseWheelMoved(MouseWheelEvent e)
        {
            if (isEnabled())
            {
                wheelLayout.rotateWheel(e.getWheelRotation());
            }
        }
    }

    /**
     * The listener that will be attached to the component
     */
    private final Listener listener = new Listener();

    /**
     * Whether this control is currently enabled
     */
    private boolean enabled = true;
    
    /**
     * Creates a new WheelLayoutControl that controls the given layout.
     * 
     * @param wheelLayout The controlled layout
     */
    public WheelLayoutControl(WheelLayout3D wheelLayout)
    {
        this.wheelLayout = wheelLayout;
    }
    

    @Override
    public void attachTo(JComponent component)
    {
        component.addMouseWheelListener(listener);
    }

    @Override
    public void detachFrom(JComponent component)
    {
        component.removeMouseWheelListener(listener);
    }

    @Override
    public void setEnabled(boolean enabled)
    {
        this.enabled = enabled;
    }

    @Override
    public boolean isEnabled()
    {
        return enabled;
    }
}


