/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2012 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.rendering;

import javax.media.opengl.GL3;

import de.javagl.swogl.PickingRayComputer;
import de.javagl.swogl.SwoglRenderer;
import de.javagl.swogl.SwoglSetup;

/**
 * Implementation of a {@link SwoglSetup} using JOGL.
 */
public class SwoglSetupShaderJOGL implements SwoglSetup
{
    /**
     * The {@link SwoglRenderer}of this setup.
     */
    private final SwoglRendererShaderJOGL swoglRenderer;
    
    /**
     * The {@link PickingRayComputer} of this setup
     */
    private final PickingRayComputerShaderJOGL pickingRayComputer;
    
    /**
     * Creates a new SwoglSetupShaderJOGL
     */
    public SwoglSetupShaderJOGL()
    {
        this.pickingRayComputer = new PickingRayComputerShaderJOGL();
        this.swoglRenderer = new SwoglRendererShaderJOGL();
    }
    
    /**
     * Set the current GL for the pickingRayComputer and the
     * swoglRenderer
     * 
     * @param gl The current GL
     */
    public void setGL(GL3 gl)
    {
        pickingRayComputer.setGL(gl);
        swoglRenderer.setGL(gl);
    }
    
    @Override
    public SwoglRendererShaderJOGL getSwoglComponentHandler()
    {
        return swoglRenderer;
    }

    @Override
    public PickingRayComputerShaderJOGL getPickingRayComputer()
    {
        return pickingRayComputer;
    }

}
