/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2012 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.scene;

import java.util.List;

/**
 * Interface for all classes that want to be informed when
 * a {@link LightSetup} changed
 */
public interface LightSetupListener
{
    /**
     * Will be called when the given {@link LightSetup} changed.
     * The given lists may be empty, in case that a property
     * of the {@link LightSetup} (like the ambient light color)
     * has changed
     * 
     * @param lightSetup The light setup 
     * @param addedLights An unmodifiable list of the lights that 
     * have been added
     * @param removedLights An unmodifiable list of the lights that
     * have been removed
     */
    void lightSetupChanged(LightSetup lightSetup, 
        List<Light> addedLights, List<Light> removedLights);
    
}
