#version 330 core

#define MAX_LIGHTS 8

#define LIGHT_TYPE_DIRECTIONAL 0
#define LIGHT_TYPE_POINT 1
#define LIGHT_TYPE_SPOT 2

// The attributes of the object, stored in VBOs. 
// Namely the position, normal and texCoords
in vec3 inPosition;
in vec3 inNormal;
in vec2 inTexCoord;

//The matrices
uniform mat4 modelMatrix;
uniform mat4 normalMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;

//The light structure. The position and spotDirection
//are given in view space
struct Light
{
    int type;
    
    vec4 ambient;
    vec4 diffuse;
    vec4 specular;
    vec4 position;
    
    vec3 spotDirection;
    float spotExponent;
    float spotCutoff;
    
    float constantAttenuation;
    float linearAttenuation;
    float quadraticAttenuation;
};

// The lights
uniform Light lights[MAX_LIGHTS];
uniform int numLights;  

uniform vec4 globalAmbient;


//The material structure
struct Material 
{
    vec4 ambient;
    vec4 diffuse;
    vec4 specular;
    vec4 emission;
    float shininess;
};

// The material
uniform Material material;

// Position, normal and texture coordinates for fragment shader
out vec3 fragmentPosition;
out vec3 fragmentNormal;
out vec2 fragmentTexCoord;

void main (void)
{
    // Compute the eye-coordinate position of the vertex
    mat4 modelviewMatrix = viewMatrix*modelMatrix;
    vec4 position = vec4(inPosition, 1.0);
    vec4 fragmentPosition4 = modelviewMatrix * position; 
    fragmentPosition = fragmentPosition4.xyz / fragmentPosition4.w;

    // Compute the transformed normal
    fragmentNormal = (normalMatrix*vec4(inNormal, 0.0)).xyz;

    // Pass the texture coordinates to the fragment shader
    fragmentTexCoord = inTexCoord;
    
    // Do fixed functionality vertex transform
    gl_Position = projectionMatrix*modelviewMatrix*position;
}



